# This Python file uses the following encoding: utf-8
import numpy as np
from PIL import Image
from scipy import misc


class CypressCommonHelper:
    def __init__(self):
        pass

    @staticmethod
    def np_bytes_to_np_array(np_bytes, axis_0, axis_1):
        """
        Convert a flatterned numpy array to multi-dimensional numpy array. This method is used to reconstruct RGB image
           numpy array from Redis record. Caller can use cv2.imwrite(nparray) to get the image file.
        :param np_bytes: flattened numpy.ndarray.
        :param axis_0: dimension 0
        :param axis_1: dimension 1
        :return: multi-dimensional numpy array of a image
        """
        try:
            return np.frombuffer(np_bytes, dtype=np.uint8).reshape(axis_0, axis_1, 3)
        except:
            return "cannot convert flatterned numpy array"

    @staticmethod
    def np_img_to_np_bytes(np_img):
        """
        Flattern an image Numpy array to an one-dimensional numpy array. The input image can be got by cv2.imread(<img file>)
        :param np_img: numpy array of image.
        :return: np_array_bytes, axis_0, axis_1
        """
        try:
            axis_0, axis_1, _ = np_img.shape
            np_array_bytes = np_img.tobytes()
            return np_array_bytes, axis_0, axis_1
        except:
            return "cannot flat numpy array to numpy array bytes"

    @staticmethod
    def img_rgb2bgr(image_rgb):
        """
        Convert a 'RGB' image to a 'BGR' image.
        :param image_rgb: the image object that is in 'RGB' or 'RGBA' mode
        :type image_rgb: `~PIL.Image.Image` object
        :return: the image object in 'BGR' mode
        :rtype: `~PIL.Image.Image` object
        """
        if image_rgb.mode == 'RGBA':
            r, g, b, _ = image_rgb.split()
        elif image_rgb.mode == 'RGB':
            r, g, b = image_rgb.split()
        else:
            raise IOError('Input image is either RGBA or RGB image.')

        image_bgr = Image.merge("RGB", (b, g, r))
        return image_bgr

    @staticmethod
    def np_bgr2rgb(np_bgr):
        """
        Convert a 'BGR' image numpy array to a 'RGB' image numpy array.
        :param image_np: the numpy array of the image in mode 'BGR'
        :type image_np: numpy array
        :return: numpy array of the image in mode 'RGB'
        :rtype: numpy array
        """
        image_bgr = misc.toimage(np_bgr)
        b, g, r = image_bgr.split()
        image_rgb = Image.merge("RGB", (r, g, b))
        return misc.fromimage(image_rgb)

    @staticmethod
    def img_rgb_2_np_bgr(image_rgb):
        """
        Convert a 'RGB' image object to a 'BGR' image numpy array.
        :param image_rgb: the image object that is in 'RGB' or 'RGBA' mode
        :type image_rgb: `~PIL.Image.Image` object
        :return: numpy array of the image in mode 'BGR'
        :rtype: numpy array
        """
        if image_rgb.mode == 'RGBA':
            r, g, b, _ = image_rgb.split()
        elif image_rgb.mode == 'RGB':
            r, g, b = image_rgb.split()
        else:
            raise IOError('Input image is either RGBA or RGB image.')

        image_bgr = Image.merge("RGB", (b, g, r))
        np_bgr = misc.fromimage(image_bgr)
        return np_bgr



import json
import logging
import socket
import traceback
from time import sleep
import confluent_kafka


class KafkaHelper:
    __logger = logging.getLogger(name="KafkaHelper")

    def __init__(self):
        pass

    @staticmethod
    def try_connect_kafka(bootstrap_server, is_consumer=False, topic=None, group="group1", partition=0):
        """
        This function tries to connect to a kafka pipe. If success, it returns a Kafka client object; otherwise,
        it returns None.
        :param bootstrap_server: bootstrap server url. The bootstrap server should be kafka broker.
        e.g. <hostname>:<port>

        :param is_consumer: is a consumer connection. default: False
        :param topic: kafka topic. only needed when is_consumer is set to True.
        :param group: consumer group, default to group1. only needed when is_consumer is set to True.
        :param partition: consumer partition, default to 0. only needed when is_consumer is set to True.
        :return: on success: a kafka consumer or producer object. on failure: it returns None
        """
        kafka_client = None
        retry = 100
        while retry >= 0:
            if kafka_client:
                break
            retry -= 1
            try:
                if is_consumer:
                    consumer_config = {'bootstrap.servers': bootstrap_server,
                                       'group.id': group,
                                       # 'client.id': socket.gethostname(),
                                       'max.partition.fetch.bytes': 10485760,
                                       # 'default.topic.config': {'auto.offset.reset': 'smallest'},
                                       'session.timeout.ms': 8000}
                    kafka_client = confluent_kafka.Consumer(**consumer_config)
                    # use partition for consume messages. The partition number is determined by the client io
                    kafka_client.assign([confluent_kafka.cimpl.TopicPartition(topic, partition)])
                else:
                    # producer_config = {'bootstrap.servers': bootstrap_server,
                    #                    'client.id': socket.gethostname(),
                    #                    'queue.buffering.max.messages': 1,
                    #                    'queue.buffering.max.ms': 1,
                    #                    'default.topic.config': {'produce.offset.report': True}}
                    # kafka_client = confluent_kafka.Producer(**producer_config)

                    from kafka import KafkaProducer
                    kafka_client = KafkaProducer(bootstrap_servers=bootstrap_server, client_id=socket.gethostname(),
                                                 value_serializer=lambda v: json.dumps(v).encode('utf-8'), retries=2,
                                                 max_block_ms=1)
            # TODO: catch different types of exception, dealing with different strategies
            except Exception as ex:
                KafkaHelper.__logger.error(traceback.format_exc())
                # wait 0.1 second to retry
                sleep(0.1)
                kafka_client = None
        return kafka_client

    @staticmethod
    def send_message(kafka_producer, topic, msg, bootstrap_server=None, unittest=False):
        """
        Send out message to the specific topic, with the task uuid
        :param kafka_producer: a kafka producer instance
        :param topic: producer topic
        :param msg: message to be sent through kafka pipe. It is a json object.
        :type msg: json object. It has a mandatory attribute: task_id
        """

        # send a message to the topic
        while True:  # do a for loop until the message is send out
            if kafka_producer is None:
                if bootstrap_server is None:
                    raise Exception("kafka producer is not initalized, missing bootstrap_server")
                kafka_producer = KafkaHelper.try_connect_kafka(bootstrap_server)
            try:
                if unittest:
                    sleep(2)
                evt_sent = kafka_producer.send(topic, msg).get(timeout=1)
                kafka_producer.flush()

                KafkaHelper.__logger.info("message sent to: " + topic)
                if evt_sent > 0:
                    break
                # if the message is not sent out, create a new producer.
                if bootstrap_server:
                    kafka_producer = None
            except Exception as ex:
                KafkaHelper.__logger.error(traceback.format_exc())

